;========================================================================
; Animated dialog demo, modal dialog as main.
;========================================================================

include \GeneSys\include\GeneSysRt.inc

;========================================================================

.data

  hInstance     HINSTANCE 0

  szDialogName  db "MyDialog",0

.code

;========================================================================
; This is the "callback" function that processes messages sent to the
; dialog. In most cases, your code should return zero whether it
; processes a message or not. See the API documentation for details.
;========================================================================

DialogProc proc hWndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL rcDlg :RECT
    LOCAL rcwa  :RECT

    .IF uMsg == WM_INITDIALOG

      ;-----------------------------------------------
      ; The WM_INITDIALOG message is sent immediately
      ; after the dialog is displayed.
      ;-----------------------------------------------

      ;-----------------------------------------------------------
      ; Calculate the correct coordinates and move the dialog so
      ; it will appear above the task bar at the right edge of
      ; the desktop. For this simple coding to work correctly the
      ; taskbar must be set for Always on top. If it is not, the
      ; dialog will appear at the bottom of the taskbar.
      ;-----------------------------------------------------------

      invoke GetWindowRect, hWndDlg, ADDR rcDlg
      invoke SystemParametersInfo, SPI_GETWORKAREA, 0, ADDR rcwa, 0
      invoke GetSystemMetrics, SM_CXSCREEN
      sub eax, rcDlg.right
      mov edx, rcwa.bottom
      sub edx, rcDlg.bottom
      invoke MoveWindow, hWndDlg,
                         eax,
                         edx,
                         rcDlg.right,
                         rcDlg.bottom,
                         TRUE

      ;-------------------------------------------------------
      ; Set a timer that will fire (expire) after 3 seconds,
      ; and animate the dialog from bottom to top using slide
      ; animation.
      ;-------------------------------------------------------

      invoke SetTimer, hWndDlg, 1, 3000, NULL
      invoke AnimateWindow, hWndDlg, 200, AW_SLIDE or AW_VER_NEGATIVE

    .ELSEIF uMsg == WM_TIMER

      ;------------------------------------------------------------
      ; The WM_TIMER message is sent when a timer fires (expires).
      ;------------------------------------------------------------

      ;------------------------------------------------------------
      ; Destroy the timer, animate the dialog from top to bottom
      ; using slide animation, specifying AW_HIDE so the animation
      ; will take effect, and close the dialog.
      ;------------------------------------------------------------

      invoke KillTimer, hWndDlg, 1

      invoke AnimateWindow, hWndDlg,
                            200,
                            AW_SLIDE or AW_HIDE or AW_VER_POSITIVE

      invoke EndDialog, hWndDlg, NULL

    .ELSEIF uMsg == WM_CLOSE

      ;-------------------------------------------------------
      ; The WM_CLOSE message is sent when the user clicks the
      ; Close button (at the right end of the title bar), or
      ; selects Close on the Window menu, or presses Alt+F4.
      ;-------------------------------------------------------

      ;------------------------------------------
      ; A modal dialog is closed (and destroyed)
      ; with the EndDialog function.
      ;------------------------------------------

      invoke EndDialog, hWndDlg, NULL

    .ENDIF

    xor eax, eax
    ret

DialogProc endp


;========================================================================

start:

    invoke GetModuleHandle, NULL
    mov hInstance, eax

    ;---------------------------------------------------
    ; Create a modal dialog from the template resource.
    ; DialogBoxParam will not return until the dialog
    ; is destroyed.
    ;---------------------------------------------------

    invoke DialogBoxParam, hInstance,
                           ADDR szDialogName,
                           NULL,
                           ADDR DialogProc,
                           NULL

    invoke ExitProcess, eax

;========================================================================

end start
