;========================================================================
; DrawText demo, application window as main.
;========================================================================

include \GeneSys\include\GeneSysRt.inc

;========================================================================

.data

  hFont         dd 0

  szDisplayName db "DrawText Demo",0
  szClassName   db "drawtext_demo_class",0
  szFontName    db "Courier New",0
  szError       db "CreateFont failed",0
  szHello       db "Hello World!",0

.code

;========================================================================

WinMain proc hInstance:HINSTANCE,
             hPrevInstance:HINSTANCE,
             lpCmdLine:LPSTR,
             nCmdShow:DWORD

    LOCAL hWnd  :HWND
    LOCAL wMsg  :MSG
    LOCAL x     :DWORD
    LOCAL y     :DWORD
    LOCAL wcx   :WNDCLASSEX

    ;------------------------------------
    ; Populate the WNDCLASSEX structure.
    ;------------------------------------

    mov wcx.cbSize, SIZEOF WNDCLASSEX

    ;----------------------------------------------------
    ; Specify the CS_HREDRAW and CS_VREDRAW class styles
    ; so the window will be redrawn when it is resized.
    ;----------------------------------------------------

    mov wcx.style, CS_HREDRAW or CS_VREDRAW

    mov wcx.lpfnWndProc, OFFSET WindowProc
    mov wcx.cbClsExtra, NULL
    mov wcx.cbWndExtra, NULL
    push hInstance
    pop wcx.hInstance
    invoke LoadIcon, NULL, IDI_APPLICATION
    mov wcx.hIcon, eax
    invoke LoadCursor, NULL, IDC_ARROW
    mov wcx.hCursor, eax

    ;-------------------------------------------------------------
    ; The color value for the window background brush, as well as
    ; as the text and text background colors below, are specified
    ; as COLORREF values of the hexadecimal form 00BBGGRR, where
    ; the bytes specify the relative intensities of blue, green,
    ; and red.
    ;-------------------------------------------------------------

    invoke CreateSolidBrush, 00ff0000h
    mov wcx.hbrBackground, eax

    mov wcx.lpszMenuName, NULL
    mov wcx.lpszClassName, OFFSET szClassName
    mov wcx.hIconSm, 0

    ;----------------------------------
    ; Register the class with Windows.
    ;----------------------------------

    invoke RegisterClassEx, ADDR wcx

    ;-----------------------------------------
    ; Specify the window size and calc the
    ; coordinates to center it on the screen.
    ;-----------------------------------------

    W = 400
    H = 300
    invoke GetSystemMetrics, SM_CXSCREEN
    shr eax, 1
    sub eax, W / 2
    mov x, eax
    invoke GetSystemMetrics, SM_CXSCREEN
    shr eax, 1
    sub eax, H / 2
    mov y, eax

    ;----------------------------------------------------------------
    ; Create the window and save the handle. The WS_OVERLAPPEDWINDOW
    ; style will create a resizable window with a title bar, border,
    ; system menu, and minimize, maximize, and close buttons.
    ;----------------------------------------------------------------

    invoke CreateWindowEx, NULL,
                           ADDR szClassName,
                           ADDR szDisplayName,
                           WS_OVERLAPPEDWINDOW,
                           x, y, W, H,
                           NULL, NULL,
                           hInstance, NULL
    mov hWnd, eax

    ;-----------------------------------------
    ; Set the window's show state and send it
    ; its first WM_PAINT message.
    ;-----------------------------------------

    invoke ShowWindow, hWnd, nCmdShow
    invoke UpdateWindow, hWnd

    ;-----------------------------------------------------------
    ; This is a minimal message loop for an application window.
    ; GetMessage retrieves messages from the calling threads
    ; message queue, returning zero when it retrieves a WM_QUIT
    ; message. If no message is available GetMessage waits for
    ; one to be posted before it returns. TranslateMessage
    ; translates virtual key messages to character messages.
    ; DispatchMessage dispatches the message to the window
    ; procedure.
    ;-----------------------------------------------------------

  msgloop:

    invoke GetMessage, ADDR wMsg, NULL, 0, 0
    .IF eax != 0
      invoke TranslateMessage, ADDR wMsg
      invoke DispatchMessage, ADDR wMsg
      jmp msgloop
    .ENDIF

    ;--------------------------------------------------------
    ; Return the exit code, specified in the PostQuitMessage
    ; call in WindowProc, to the system.
    ;--------------------------------------------------------

    mov eax, wMsg.wParam
    ret

WinMain endp

;========================================================================
; This is the "callback" function that processes messages sent to the
; window. In most cases, your code should return zero whether it
; processes a message or not. See the API documentation for details.
; Any messages not processed should normally be passed to the default
; window procedure, and then whatever it returns should be returned
; to the system.
;========================================================================

WindowProc proc hWnd:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL hgdiobj :HGDIOBJ
    LOCAL ps      :PAINTSTRUCT
    LOCAL rc      :RECT

    .IF uMsg == WM_CREATE

      ;---------------------------------------------------
      ; The WM_CREATE message is sent after the window is
      ; created, and before it is made visible.
      ;---------------------------------------------------

      ;----------------------------------------------------
      ; Create the font and save the handle. The height is
      ; specified as 50 pixels. The width, in parameter 2,
      ; is specfied as zero, so the system will determine
      ; the proper width. For the other parameters, "0" was
      ; used instead of "NULL" or "FALSE" to avoid a line
      ; too long error.
      ;----------------------------------------------------

      invoke CreateFont, 50,
                         0,
                         0,
                         0,
                         FW_BOLD,
                         0,
                         0,
                         0,
                         DEFAULT_CHARSET,
                         OUT_TT_PRECIS,
                         CLIP_DEFAULT_PRECIS,
                         PROOF_QUALITY,
                         DEFAULT_PITCH or FF_DONTCARE,
                         ADDR szFontName

      mov hFont, eax

      ;-----------------------------------------------
      ; Display an error message if CrateFont failed.
      ;-----------------------------------------------

      .IF hFont == 0
        invoke MessageBox, hWnd, ADDR szError, 0, 0
      .ENDIF

    .ELSEIF uMsg == WM_PAINT

      ;-------------------------------------------------------
      ; The WM_PAINT message is sent, by the system and only
      ; by the system, when the window needs to be repainted.
      ;-------------------------------------------------------

      ;-----------------------
      ; Prepare for painting.
      ;-----------------------

      invoke BeginPaint, hWnd, ADDR ps

      ;--------------------------------------------------
      ; Get the coordinates of our window's client area.
      ;--------------------------------------------------

      invoke GetClientRect, hWnd, ADDR rc

      ;-----------------------------------------------
      ; Select the font into the device context and
      ; save the handle to the object being replaced.
      ;-----------------------------------------------

      invoke SelectObject, ps.hdc, hFont
      mov hgdiobj, eax

      ;---------------------------------------------------------
      ; Set the text background color to match the background
      ; color of the window, and the foreground color to white.
      ;---------------------------------------------------------

      invoke SetBkColor, ps.hdc, 0ff0000h
      invoke SetTextColor, ps.hdc, 0ffffffh

      ;-------------------------------------------
      ; Draw the text on a single line, centering
      ; it horizontally and vertically.
      ;-------------------------------------------

      invoke DrawText, ps.hdc,
                       ADDR szHello,
                       -1,
                       ADDR rc,
                       DT_SINGLELINE or DT_CENTER or DT_VCENTER

      ;----------------------------------------------------------
      ; Select the original object back into the device context.
      ;----------------------------------------------------------

      invoke SelectObject, ps.hdc, hgdiobj

      ;---------------
      ; End painting.
      ;---------------

      invoke EndPaint, hWnd, ADDR ps

    .ELSEIF uMsg == WM_CLOSE

      ;-------------------------------------------------------
      ; The WM_CLOSE message is sent when the user clicks the
      ; Close button (at the right end of the title bar), or
      ; selects Close on the Window menu, or presses Alt+F4.
      ;-------------------------------------------------------

      ;-----------------------------------------------------
      ; Free the system resources associated with the font.
      ;-----------------------------------------------------

      invoke DeleteObject, hFont

      ;------------------------------------------------------
      ; Destroy the window. DestroyWindow will automatically
      ; destroy any child windows (e.g controls) before it
      ; destroys the parent.
      ;------------------------------------------------------

      invoke DestroyWindow, hWnd

    .ELSEIF uMsg == WM_DESTROY

      ;---------------------------------------------------------
      ; This message is sent when the window is being destroyed,
      ; and it must be handled to ensure that the application
      ; exits properly.
      ;
      ; PostQuitMessage posts a WM_QUIT message to the message
      ; queue.
      ;---------------------------------------------------------

      invoke PostQuitMessage, NULL

    .ELSE

      invoke DefWindowProc, hWnd, uMsg, wParam, lParam
      ret

    .ENDIF

    xor eax, eax
    ret

WindowProc endp

;========================================================================

start:

    ;-------------------------------------------------------------
    ; This code sets up and calls WinMain, passing it essentially
    ; the same values that the RTL would pass in a C application.
    ; A null is passed in the hPrevInstance parameter because it
    ; is obsolete.
    ;-------------------------------------------------------------

    invoke GetCommandLine
    push eax
    invoke GetModuleHandle, NULL
    pop edx
    invoke WinMain, eax, NULL, edx, SW_SHOWDEFAULT
    invoke ExitProcess, 0

;========================================================================

end start
