;========================================================================
; Heapwalk demo based on the CRT function, modal dialog as main.
;
; This also demonstrates sizing a control to fill the client
; area of the window.
;========================================================================

include \GeneSys\include\GeneSysRt.inc

IDC_LIST equ 101

;----------------------------------------------------------
; The CRT _heapwalk function returns heap information in a
; structure of type _HEAPINFO. The first six constants are
; used to interpret the returned status, and the final two
; to interpret the value in the _useflag member.
;----------------------------------------------------------

_HEAPINFO struct
  _pentry   dd ?
  _size     dd ?
  _useflag  dd ?
_HEAPINFO ends

_HEAPEMPTY    equ -1
_HEAPOK       equ -2
_HEAPBADBEGIN equ -3
_HEAPBADNODE  equ -4
_HEAPEND      equ -5
_HEAPBADPTR   equ -6
_FREEENTRY    equ 0
_USEDENTRY    equ 1

;========================================================================

.data

  hInstance     HINSTANCE 0
  hi            _HEAPINFO <>

  szDialogName  db "MyDialog",0
  szFormatUsed  db "USED  %X  %d",0
  szFormatFree  db "FREE  %X  %d",0

.code

;========================================================================
; This is the "callback" function that processes messages sent to the
; dialog. In most cases, your code should return zero whether it
; processes a message or not. See the API documentation for details.
;========================================================================

DialogProc proc hWndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL hWndList    :HWND
    LOCAL rc          :RECT
    LOCAL buffer[100] :BYTE

    .IF uMsg == WM_INITDIALOG

      ;-----------------------------------------------
      ; The WM_INITDIALOG message is sent immediately
      ; after the dialog is displayed.
      ;-----------------------------------------------

      ;-----------------------------------------------------
      ; Size the listbox to fill the client area. Since the
      ; dialog is not user sizable, this sizing needs to be
      ; done only once.
      ;-----------------------------------------------------

      invoke GetDlgItem, hWndDlg, IDC_LIST
      mov hWndList, eax
      invoke GetClientRect, hWndDlg, ADDR rc
      invoke MoveWindow, hWndList, 0, 0, rc.right, rc.bottom, TRUE

      ;---------------------------------------------------
      ; Walk the heap, and for each entry create a string
      ; listing the used/free status, address, and size,
      ; and add the string to the list box.
      ;---------------------------------------------------

    @@:
      invoke _heapwalk, ADDR hi
      cmp eax, _HEAPOK
      jne @F
      .IF hi._useflag == _USEDENTRY
        mov eax, OFFSET szFormatUsed
      .ELSE
        mov eax, OFFSET szFormatFree
      .ENDIF
      invoke sprintf, ADDR buffer, eax, hi._pentry, hi._size
      invoke SendMessage, hWndList, LB_ADDSTRING, 0, ADDR buffer
      jmp @B
    @@:
      mov eax, TRUE
      ret

    .ELSEIF uMsg == WM_COMMAND

      ;---------------------------------------------------------
      ; The WM_COMMAND message is sent when a control sends a
      ; notification, an accelerator is translated, or the user
      ; selects a menu command. The identifier of the menu item,
      ; control, or accelerator, is passed in the low-order word
      ; of wParam, the notification code in the high-order word
      ; of wParam, and the handle to the control in lParam.
      ;---------------------------------------------------------

      ;-----------------------------------------------------
      ; For a dialog, a WM_COMMAND message with an IDCANCEL
      ; identifier in the low-order word of wParam is sent
      ; when the user presses the escape key, regardless of
      ; whether or not the dialog has a control with this
      ; identifier.
      ;-----------------------------------------------------

      ;------------------------------------------
      ; A modal dialog is closed (and destroyed)
      ; with the EndDialog function.
      ;------------------------------------------

      mov eax, wParam
      .IF ax == IDCANCEL

        invoke EndDialog, hWndDlg, NULL

      .ENDIF

    .ELSEIF uMsg == WM_CLOSE

      ;-------------------------------------------------------
      ; The WM_CLOSE message is sent when the user clicks the
      ; Close button (at the right end of the title bar), or
      ; selects Close on the Window menu, or presses Alt+F4.
      ;-------------------------------------------------------

      invoke EndDialog, hWndDlg, NULL

    .ENDIF

    xor eax, eax
    ret

DialogProc endp

;========================================================================

start:

    invoke GetModuleHandle, NULL
    mov hInstance, eax

    ;---------------------------------------------------
    ; Create a modal dialog from the template resource.
    ; DialogBoxParam will not return until the dialog
    ; is destroyed.
    ;---------------------------------------------------

    invoke DialogBoxParam, hInstance,
                           ADDR szDialogName,
                           NULL,
                           ADDR DialogProc,
                           NULL

    invoke ExitProcess, eax

;========================================================================

end start
