;========================================================================
; Menu demo, with keyboard accelerators, modeless dialog as main.
;
; Keyboard accelerators are easier to implement in a modeless
; dialog because you have direct access to the message loop.
;========================================================================

include \GeneSys\include\GeneSysRt.inc

IDM_NEW       equ 101
IDM_OPEN      equ 102
IDM_SAVE      equ 103
IDM_SAVEAS    equ 104
IDM_PAGESETUP equ 105
IDM_PRINT     equ 106
IDM_EXIT      equ 107
IDM_ABOUT     equ 108

;========================================================================

.data

  szDialogName  db "MyDialog",0
  szMenuName    db "MyMenu",0
  szAccelName   db "MyAccel",0
  szCaption     db "Menu",0
  szNew         db "New",0
  szOpen        db "Open",0
  szSave        db "Save",0
  szSaveAs      db "SaveAs",0
  szPageSetup   db "PageSetup",0
  szPrint       db "Print",0
  szAbout       db "About",0

  hInstance     HINSTANCE 0
  hDlg          HWND      0
  hAccel        HACCEL    0
  msg           MSG       <>

.code

;========================================================================
; This is the "callback" function that processes messages sent to the
; dialog. In most cases, your code should return zero whether it
; processes a message or not. See the API documentation for details.
;========================================================================

DialogProc proc hWndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    .IF uMsg == WM_INITDIALOG

      ;-----------------------------------------------
      ; The WM_INITDIALOG message is sent immediately
      ; after the dialog is displayed.
      ;-----------------------------------------------

      ;-------------------------------------------
      ; Load the menu resource from the EXE file.
      ;-------------------------------------------

      invoke LoadMenu, hInstance, ADDR szMenuName

      ;---------------------------------------
      ; Assign the menu to the dialog window.
      ;---------------------------------------

      invoke SetMenu, hWndDlg, eax

    .ELSEIF uMsg == WM_COMMAND

      ;---------------------------------------------------------
      ; The WM_COMMAND message is sent when a control sends a
      ; notification, an accelerator is translated, or the user
      ; selects a menu command. The identifier of the menu item,
      ; control, or accelerator is passed in the low-order word
      ; of wParam, the notification code in the high-order word
      ; of wParam, and the handle to the control in lParam.
      ;---------------------------------------------------------

      mov eax, wParam

      .IF ax == IDM_NEW

        invoke MessageBox, hWndDlg, ADDR szNew, ADDR szCaption, 0

      .ELSEIF ax == IDM_OPEN

        invoke MessageBox, hWndDlg, ADDR szOpen, ADDR szCaption, 0

      .ELSEIF ax == IDM_SAVE

        invoke MessageBox, hWndDlg, ADDR szSave, ADDR  szCaption, 0

      .ELSEIF ax == IDM_SAVEAS

        invoke MessageBox, hWndDlg, ADDR szSaveAs, ADDR szCaption, 0

      .ELSEIF ax == IDM_PAGESETUP

        invoke MessageBox, hWndDlg, ADDR szPageSetup, ADDR szCaption, 0

      .ELSEIF ax == IDM_PRINT

        invoke MessageBox, hWndDlg, ADDR szPrint, ADDR  szCaption, 0

      .ELSEIF ax == IDM_ABOUT

        invoke MessageBox, hWndDlg, ADDR szAbout, ADDR  szCaption, 0

      .ELSEIF ax == IDM_EXIT

        invoke DestroyWindow, hWndDlg

      .ELSEIF ax == IDCANCEL

        ;-----------------------------------------------------
        ; For a dialog, a WM_COMMAND message with an IDCANCEL
        ; identifier in the low-order word of wParam is sent
        ; when the user presses the escape key, regardless of
        ; whether or not the dialog has a control with this
        ; identifier.
        ;-----------------------------------------------------

        invoke DestroyWindow, hWndDlg

      .ENDIF

    .ELSEIF uMsg == WM_CLOSE

      ;-------------------------------------------------------
      ; The WM_CLOSE message is sent when the user clicks the
      ; Close button (at the right end of the title bar), or
      ; selects Close on the Window menu, or presses Alt+F4.
      ;-------------------------------------------------------

      ;------------------------------------------------------------
      ; A modeless dialog is destroyed like a normal application
      ; window, starting with a call to the DestroyWindow function.
      ; DestroyWindow will automatically destroy  any child windows
      ; (e.g. controls) before it destroys the parent.
      ;------------------------------------------------------------

      invoke DestroyWindow, hWndDlg

    .ELSEIF uMsg == WM_DESTROY

      ;----------------------------------------------------------
      ; This message is sent when the window is being destroyed,
      ; and it must be handled to ensure that the application
      ; exits properly.
      ;
      ; PostQuitMessage posts a WM_QUIT message to the message
      ; queue.
      ;----------------------------------------------------------

      invoke PostQuitMessage, NULL

    .ENDIF

    xor eax, eax
    ret

DialogProc endp

;========================================================================

start:

    invoke GetModuleHandle, NULL
    mov hInstance, eax

    ;---------------------------------------------
    ; Create a modeless dialog from the template
    ; resource and save the dialog window handle.
    ;---------------------------------------------

    invoke CreateDialogParam, hInstance,
                              ADDR szDialogName,
                              NULL,
                              ADDR DialogProc,
                              NULL
    mov hDlg, eax

    ;-------------------------------------------------
    ; Load the accelerator table and save the handle.
    ;-------------------------------------------------

    invoke LoadAccelerators, hInstance, ADDR szAccelName
    mov hAccel, eax

    ;---------------------------------------------------------
    ; This is a special message loop for a modeless dialog.
    ; As with the message loop for an application window,
    ; GetMessage retrieves messages from the calling threads
    ; message queue, returning zero when it retrieves a
    ; WM_QUIT message. If no message is available GetMessage
    ; waits for one to be posted before it returns. Unlike a
    ; message loop for an application window, this message
    ; loop depends on IsDialogMessage to process the messages,
    ; instead of TranslateMessage and DispatchMessage. The
    ; advantage of using IsDialogMessage is that it provides
    ; dialog-style keyboard navigation that would otherwise
    ; require a considerably amount of code to implement.
    ; The special feature of this message loop is that it
    ; calls TranslateAccelerator to process the accelerator
    ; keys, and then calls IsDialogMessage only if
    ; TranslateAccelerator does not process the message.
    ;---------------------------------------------------------

  msgLoop:

    invoke GetMessage, ADDR msg, 0, 0, 0
    .IF eax != 0
      invoke TranslateAccelerator, hDlg, hAccel, ADDR msg
      .IF eax == 0
        invoke IsDialogMessage, hDlg, ADDR msg
      .ENDIF
      jmp msgLoop
    .ENDIF

    invoke ExitProcess, eax

;========================================================================

end start