;========================================================================
; System color demo, modal dialog as main.
;========================================================================

include \GeneSys\include\GeneSysRt.inc
include \GeneSys\macros\macros.asm

IDC_LIST   equ 101
IDC_SAMPLE equ 102
IDC_HEXVAL equ 103

;------------------------------------------------------------------
; This macro is used to simplify the task of loading the 36 system
; color constant names into the list box. The reparg macro allows
; this macro to accept the names as quoted strings.
;------------------------------------------------------------------

LBADDS MACRO h,s
  invoke SendMessage, h, LB_ADDSTRING, 0, reparg(s)
ENDM

;========================================================================

.data

  hInstance     HINSTANCE 0
  hWndList      HWND 0
  hWndSample    HWND 0
  hBrush        HBRUSH 0

  ;-----------------------------------------------------
  ; This is a table of system color type constants that
  ; will be used to associate the color types with the
  ; list box items.
  ;-----------------------------------------------------

  colorTypes    dd COLOR_3DDKSHADOW
                dd COLOR_3DFACE
                dd COLOR_3DHIGHLIGHT
                dd COLOR_3DHILIGHT
                dd COLOR_3DLIGHT
                dd COLOR_3DSHADOW
                dd COLOR_ACTIVEBORDER
                dd COLOR_ACTIVECAPTION
                dd COLOR_APPWORKSPACE
                dd COLOR_BACKGROUND
                dd COLOR_BTNFACE
                dd COLOR_BTNHIGHLIGHT
                dd COLOR_BTNHILIGHT
                dd COLOR_BTNSHADOW
                dd COLOR_BTNTEXT
                dd COLOR_CAPTIONTEXT
                dd COLOR_DESKTOP
                dd COLOR_GRADIENTACTIVECAPTION
                dd COLOR_GRADIENTINACTIVECAPTION
                dd COLOR_GRAYTEXT
                dd COLOR_HIGHLIGHT
                dd COLOR_HIGHLIGHTTEXT
                dd COLOR_HOTLIGHT
                dd COLOR_INACTIVEBORDER
                dd COLOR_INACTIVECAPTION
                dd COLOR_INACTIVECAPTIONTEXT
                dd COLOR_INFOBK
                dd COLOR_INFOTEXT
                dd COLOR_MENU
                dd COLOR_MENUBAR
                dd COLOR_MENUHILIGHT
                dd COLOR_MENUTEXT
                dd COLOR_SCROLLBAR
                dd COLOR_WINDOW
                dd COLOR_WINDOWFRAME
                dd COLOR_WINDOWTEXT

  buffer        db 16 dup(0)

  ;------------------------------------------------------
  ; This is the format string for the wsprintf function.
  ; It directs the function to format its argument as
  ; upper-case hex, right justified, and filled out with
  ; leading zeros to a width of 8 characters.
  ;------------------------------------------------------

  format        db "%08X",0

  szDialogName  db "MyDialog",0

.code

;========================================================================
; This is the "callback" function that processes messages sent to the
; dialog. In most cases, your code should return zero whether it
; processes a message or not. See the API documentation for details.
;========================================================================

DialogProc proc hWndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    .IF uMsg == WM_INITDIALOG

      ;-----------------------------------------------
      ; The WM_INITDIALOG message is sent immediately
      ; after the dialog is displayed.
      ;-----------------------------------------------

      invoke GetDlgItem, hWndDlg, IDC_LIST
      mov hWndList, eax
      invoke GetDlgItem, hWndDlg, IDC_SAMPLE
      mov hWndSample, eax

      LBADDS hWndList, "COLOR_3DDKSHADOW"
      LBADDS hWndList, "COLOR_3DFACE"
      LBADDS hWndList, "COLOR_3DHIGHLIGHT"
      LBADDS hWndList, "COLOR_3DHILIGHT"
      LBADDS hWndList, "COLOR_3DLIGHT"
      LBADDS hWndList, "COLOR_3DSHADOW"
      LBADDS hWndList, "COLOR_ACTIVEBORDER"
      LBADDS hWndList, "COLOR_ACTIVECAPTION"
      LBADDS hWndList, "COLOR_APPWORKSPACE"
      LBADDS hWndList, "COLOR_BACKGROUND"
      LBADDS hWndList, "COLOR_BTNFACE"
      LBADDS hWndList, "COLOR_BTNHIGHLIGHT"
      LBADDS hWndList, "COLOR_BTNHILIGHT"
      LBADDS hWndList, "COLOR_BTNSHADOW"
      LBADDS hWndList, "COLOR_BTNTEXT"
      LBADDS hWndList, "COLOR_CAPTIONTEXT"
      LBADDS hWndList, "COLOR_DESKTOP"
      LBADDS hWndList, "COLOR_GRADIENTACTIVECAPTION"
      LBADDS hWndList, "COLOR_GRADIENTINACTIVECAPTION"
      LBADDS hWndList, "COLOR_GRAYTEXT"
      LBADDS hWndList, "COLOR_HIGHLIGHT"
      LBADDS hWndList, "COLOR_HIGHLIGHTTEXT"
      LBADDS hWndList, "COLOR_HOTLIGHT"
      LBADDS hWndList, "COLOR_INACTIVEBORDER"
      LBADDS hWndList, "COLOR_INACTIVECAPTION"
      LBADDS hWndList, "COLOR_INACTIVECAPTIONTEXT"
      LBADDS hWndList, "COLOR_INFOBK"
      LBADDS hWndList, "COLOR_INFOTEXT"
      LBADDS hWndList, "COLOR_MENU"
      LBADDS hWndList, "COLOR_MENUBAR"
      LBADDS hWndList, "COLOR_MENUHILIGHT"
      LBADDS hWndList, "COLOR_MENUTEXT"
      LBADDS hWndList, "COLOR_SCROLLBAR"
      LBADDS hWndList, "COLOR_WINDOW"
      LBADDS hWndList, "COLOR_WINDOWFRAME"
      LBADDS hWndList, "COLOR_WINDOWTEXT"

      ;-------------------------------------------------------
      ; Set the current selection to the first item and send
      ; a fake LBN_SELCHANGE notification so the color sample
      ; and hex value displays will be updated.
      ;-------------------------------------------------------

      invoke SendMessage, hWndList, LB_SETCURSEL, 0, 0
      mov eax, LBN_SELCHANGE
      shl eax, 16
      invoke SendMessage, hWndDlg, WM_COMMAND, eax, 0

    .ELSEIF uMsg == WM_COMMAND

      ;---------------------------------------------------------
      ; The WM_COMMAND message is sent when a control sends a
      ; notification, an accelerator is translated, or the user
      ; selects a menu command. The identifier of the menu item,
      ; control, or accelerator, is passed in the low-order word
      ; of wParam, the notification code in the high-order word
      ; of wParam, and the handle to the control in lParam.
      ;---------------------------------------------------------

      mov eax, wParam
      shr eax, 16
      .IF eax == LBN_SELCHANGE

        ;-----------------------------------------------------
        ; The LBN_SELCHANGE notification is sent whenever the
        ; list box selection changes, and whenever it does we
        ; need to update everything to reflect the selection.
        ; The LB_GETCURSEL message retrieves the zero-based
        ; index number of the currently selected item.
        ;-----------------------------------------------------

        invoke SendMessage, hWndList, LB_GETCURSEL, 0, 0

        ;----------------------------------------------------
        ; Get the color type value that corresponds to the
        ; currently selected item. The item index number
        ; must be scaled by 4 when it is use to access the
        ; table because the table elements are 4 bytes each.
        ;
        ; The handle to the system color brush will be
        ; needed in the WM_CTLCOLORSTATIC handler.
        ;----------------------------------------------------

        mov eax, [colorTypes+eax*4]
        push eax
        invoke GetSysColorBrush, eax
        mov hBrush, eax
        pop eax
        invoke GetSysColor, eax
        invoke wsprintf, ADDR buffer, ADDR format, eax
        invoke SetDlgItemText, hWndDlg, IDC_HEXVAL, ADDR buffer

        ;----------------------------------------------------
        ; This will force an update of the sample by causing
        ; the system to send a WM_CTLCOLORSTATIC message to
        ; the dialog.
        ;----------------------------------------------------

        invoke SetDlgItemText, hWndDlg, IDC_SAMPLE, 0


      .ENDIF

      ;-----------------------------------------------------
      ; For a dialog, a WM_COMMAND message with an IDCANCEL
      ; identifier in the low-order word of wParam is sent
      ; when the user presses the escape key, regardless of
      ; whether or not the dialog has a control with this
      ; identifier.
      ;-----------------------------------------------------

      ;------------------------------------------
      ; A modal dialog is closed (and destroyed)
      ; with the EndDialog function.
      ;------------------------------------------

      mov eax, wParam
      .IF ax == IDCANCEL

        invoke EndDialog, hWndDlg, NULL

      .ENDIF

    .ELSEIF uMsg == WM_CTLCOLORSTATIC

      ;-----------------------------------------------------
      ; The WM_CTLCOLORSTATIC message will be sent whenever
      ; either of the static controls is about to be drawn.
      ; We need to return the brush that corresponds to the
      ; selected system color for the control that displays
      ; the sample, so it will be used to draw the control
      ; background, and ignore the message for control that
      ; displays the hex value of the color. The handle to
      ; the static control that is about to be drawn is
      ; passed in lParam.
      ;-----------------------------------------------------

      mov eax, hWndSample
      .IF lParam == eax

        mov eax, hBrush
        ret

      .ENDIF

    .ELSEIF uMsg == WM_CLOSE

      ;-------------------------------------------------------
      ; The WM_CLOSE message is sent when the user clicks the
      ; Close button (at the right end of the title bar), or
      ; selects Close on the Window menu, or presses Alt+F4.
      ;-------------------------------------------------------

      invoke EndDialog, hWndDlg, NULL

    .ENDIF

    xor eax, eax
    ret

DialogProc endp

;========================================================================

start:

    invoke GetModuleHandle, NULL
    mov hInstance, eax

    ;---------------------------------------------------
    ; Create a modal dialog from the template resource.
    ; DialogBoxParam will not return until the dialog
    ; is destroyed.
    ;---------------------------------------------------

    invoke DialogBoxParam, hInstance,
                           ADDR szDialogName,
                           NULL,
                           ADDR DialogProc,
                           NULL

    invoke ExitProcess, eax

;========================================================================

end start

