;========================================================================
; Translucent dialog demo, modeless dialog as main.
;
; Adapted from http://www.codeproject.com/dialog/smoothalpha.asp
;
; Requires Windows 2000 or later.
;========================================================================

include \GeneSys\include\GeneSysRt.inc

; ---------------------
; Animation constants.
; ---------------------

ANIMATION_MS      equ 250
ALPHA             equ 200
ALPHA_STEP        equ 10
ANIMATION_TIMEOUT equ ALPHA_STEP * ANIMATION_MS / ALPHA

;========================================================================

.data

  hInstance     HINSTANCE 0
  hDlg          HWND      0
  msg           MSG       <>

  szDialogName  db "MyDialog",0

.code

;========================================================================
; This is the "callback" function that processes messages sent to the
; dialog. In most cases, your code should return zero whether it
; processes a message or not. See the API documentation for details.
;========================================================================

DialogProc proc hWndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL bAlpha: DWORD

    .IF uMsg == WM_INITDIALOG

      ;-----------------------------------------------
      ; The WM_INITDIALOG message is sent immediately
      ; after the dialog is displayed.
      ;-----------------------------------------------

      ; ------------------------------------------------------------
      ; Make the dialog a layered window. GetWindowLong retrieves
      ; the extended window style information, which is then ORed
      ; with WS_EX_LAYERED to set the WS_EX_LAYERED extended window
      ; style without disturbing the other extended window style
      ; bits, and the new style is then set with SetWindowLong.
      ; ------------------------------------------------------------

      invoke GetWindowLong, hWndDlg, GWL_EXSTYLE
      or eax, WS_EX_LAYERED
      invoke SetWindowLong, hWndDlg, GWL_EXSTYLE, eax

      ; ----------------------------------------------------------
      ; Start at an alpha value of zero (completely transparent).
      ; ----------------------------------------------------------

      mov bAlpha, 0
      invoke SetLayeredWindowAttributes, hWndDlg, 0, bAlpha, LWA_ALPHA

      ; --------------------
      ; Display the dialog.
      ; --------------------

      invoke ShowWindow, hWndDlg, SW_SHOW

      ; ------------------------------------------
      ; Force an update of the client area so the
      ; controls will be included in the fade in.
      ; ------------------------------------------

      invoke RedrawWindow, hWndDlg, NULL, NULL, RDW_UPDATENOW

      ; ---------------------------------------
      ; Gradually fade the dialog in, stopping
      ; when the alpha value reaches ALPHA.
      ; ---------------------------------------

      .WHILE bAlpha < ALPHA
        invoke SetLayeredWindowAttributes, hWndDlg, 0, bAlpha, LWA_ALPHA
        invoke Sleep, ANIMATION_TIMEOUT
        add bAlpha, ALPHA_STEP
      .ENDW
      invoke SetLayeredWindowAttributes, hWndDlg, 0, ALPHA, LWA_ALPHA

    .ELSEIF uMsg == WM_COMMAND

      ;---------------------------------------------------------
      ; The WM_COMMAND message is sent when a control sends a
      ; notification, an accelerator is translated, or the user
      ; selects a menu command. The identifier of the menu item,
      ; control, or accelerator, is passed in the low-order word
      ; of wParam, the notification code in the high-order word
      ; of wParam, and the handle to the control in lParam.
      ;---------------------------------------------------------

      ;-----------------------------------------------------
      ; For a dialog, a WM_COMMAND message with an IDCANCEL
      ; identifier in the low-order word of wParam is sent
      ; when the user presses the escape key, regardless of
      ; whether or not the dialog has a control with this
      ; identifier.
      ;-----------------------------------------------------

      mov eax, wParam
      .IF ax == IDCANCEL
        invoke DestroyWindow, hWndDlg
      .ENDIF

    .ELSEIF uMsg == WM_CLOSE

      ;-------------------------------------------------------
      ; The WM_CLOSE message is sent when the user clicks the
      ; Close button (at the right end of the title bar), or
      ; selects Close on the Window menu, or presses Alt+F4.
      ;-------------------------------------------------------

      ;------------------------------------------------------------
      ; A modeless dialog is destroyed like a normal application
      ; window, starting with a call to the DestroyWindow function.
      ; DestroyWindow will automatically destroy any child windows
      ; (e.g. controls) before it destroys the parent.
      ;------------------------------------------------------------

      invoke DestroyWindow, hWndDlg

    .ELSEIF uMsg == WM_DESTROY

      ;----------------------------------------------------------
      ; This message is sent when the window is being destroyed,
      ; and it must be handled to ensure that the application
      ; exits properly.
      ;
      ; PostQuitMessage posts a WM_QUIT message to the message
      ; queue.
      ;----------------------------------------------------------

      invoke PostQuitMessage, NULL

    .ENDIF

    xor eax, eax
    ret

DialogProc endp

;========================================================================

start:

    invoke GetModuleHandle, NULL
    mov hInstance, eax

    ;---------------------------------------------
    ; Create a modeless dialog from the template
    ; resource and save the dialog window handle.
    ;---------------------------------------------

    invoke CreateDialogParam, hInstance,
                              ADDR szDialogName,
                              NULL,
                              ADDR DialogProc,
                              NULL
    mov hDlg, eax

    ;---------------------------------------------------------
    ; This is a minimal message loop for a modeless dialog.
    ; As with the message loop for an application window,
    ; GetMessage retrieves messages from the calling threads
    ; message queue, returning zero when it retrieves a
    ; WM_QUIT message. If no message is available GetMessage
    ; waits for one to be posted before it returns. Unlike
    ; a message loop for an application window, this message
    ; loop depends on IsDialogMessage to process the messages,
    ; instead of TranslateMessage and DispatchMessage. The
    ; advantage of using IsDialogMessage is that it provides
    ; dialog-style keyboard navigation that would otherwise
    ; require a considerably amount of code to implement.
    ;---------------------------------------------------------

  msgLoop:

    invoke GetMessage, ADDR msg, 0, 0, 0
    .IF eax != 0
      invoke IsDialogMessage, hDlg, ADDR msg
      jmp msgLoop
    .ENDIF

    invoke ExitProcess, eax

;========================================================================

end start
